@@ -8,7 +8,7 @@ use uv_static::EnvVars;
 pub use crate::discovery::{
     EnvironmentPreference, Error as DiscoveryError, PythonDownloads, PythonNotFound,
     PythonPreference, PythonRequest, PythonSource, PythonVariant, VersionRequest,
-    find_python_installations,
+    find_python_installations, satisfies_python_preference,
 };
 pub use crate::downloads::PlatformRequest;
 pub use crate::environment::{InvalidEnvironmentKind, PythonEnvironment};
@@ -29,19 +29,16 @@ pub use crate::version_files::{
 };
 pub use crate::virtualenv::{Error as VirtualEnvError, PyVenvConfiguration, VirtualEnvironment};
 
-mod cpuinfo;
 mod discovery;
 pub mod downloads;
 mod environment;
 mod implementation;
 mod installation;
 mod interpreter;
-mod libc;
 pub mod macos_dylib;
 pub mod managed;
 #[cfg(windows)]
 mod microsoft_store;
-pub mod platform;
 mod pointer_size;
 mod prefix;
 mod python_version;
@@ -135,7 +132,7 @@ mod tests {
     use indoc::{formatdoc, indoc};
     use temp_env::with_vars;
     use test_log::test;
-    use uv_configuration::PreviewMode;
+    use uv_configuration::Preview;
     use uv_static::EnvVars;
 
     use uv_cache::Cache;
@@ -468,7 +465,7 @@ mod tests {
                 EnvironmentPreference::OnlySystem,
                 PythonPreference::default(),
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         });
         assert!(
@@ -483,7 +480,7 @@ mod tests {
                 EnvironmentPreference::OnlySystem,
                 PythonPreference::default(),
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         });
         assert!(
@@ -508,7 +505,7 @@ mod tests {
                 EnvironmentPreference::OnlySystem,
                 PythonPreference::default(),
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         });
         assert!(
@@ -530,7 +527,7 @@ mod tests {
                 EnvironmentPreference::OnlySystem,
                 PythonPreference::default(),
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert!(
@@ -592,7 +589,7 @@ mod tests {
                 EnvironmentPreference::OnlySystem,
                 PythonPreference::default(),
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert!(
@@ -624,7 +621,7 @@ mod tests {
                 EnvironmentPreference::OnlySystem,
                 PythonPreference::default(),
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         });
         assert!(
@@ -661,7 +658,7 @@ mod tests {
                 EnvironmentPreference::OnlySystem,
                 PythonPreference::default(),
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert!(
@@ -693,7 +690,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -715,7 +712,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -741,7 +738,7 @@ mod tests {
                 EnvironmentPreference::OnlySystem,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -767,7 +764,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -790,7 +787,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
 
@@ -824,7 +821,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
 
@@ -858,7 +855,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })?;
         assert!(
@@ -880,7 +877,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })?;
         assert!(
@@ -902,7 +899,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
 
@@ -936,7 +933,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
 
@@ -973,7 +970,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             })??;
         assert!(
@@ -1004,7 +1001,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             })??;
         assert!(
@@ -1039,7 +1036,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             })??;
         assert_eq!(
@@ -1065,7 +1062,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             })??;
         assert_eq!(
@@ -1092,7 +1089,7 @@ mod tests {
                     EnvironmentPreference::OnlyVirtual,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             },
         )??;
@@ -1117,7 +1114,7 @@ mod tests {
                     EnvironmentPreference::OnlyVirtual,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             },
         )?;
@@ -1139,7 +1136,7 @@ mod tests {
                     EnvironmentPreference::OnlySystem,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             },
         )??;
@@ -1162,7 +1159,7 @@ mod tests {
                     EnvironmentPreference::OnlyVirtual,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             },
         )??;
@@ -1195,7 +1192,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             },
         )??;
@@ -1216,7 +1213,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             },
         )??;
@@ -1243,7 +1240,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
 
@@ -1261,7 +1258,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
 
@@ -1290,7 +1287,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             })??;
         assert_eq!(
@@ -1328,7 +1325,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             },
         )??;
@@ -1356,7 +1353,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             },
         )??;
@@ -1381,7 +1378,7 @@ mod tests {
                     EnvironmentPreference::ExplicitSystem,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             },
         )??;
@@ -1406,7 +1403,7 @@ mod tests {
                     EnvironmentPreference::OnlySystem,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             },
         )??;
@@ -1431,7 +1428,7 @@ mod tests {
                     EnvironmentPreference::OnlyVirtual,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             },
         )??;
@@ -1469,7 +1466,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             },
         )??;
@@ -1497,7 +1494,7 @@ mod tests {
                     EnvironmentPreference::OnlySystem,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             })??;
         assert_eq!(
@@ -1514,7 +1511,7 @@ mod tests {
                     EnvironmentPreference::OnlySystem,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             })??;
         assert_eq!(
@@ -1531,7 +1528,7 @@ mod tests {
                     EnvironmentPreference::OnlySystem,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             })?;
         assert!(
@@ -1553,7 +1550,7 @@ mod tests {
                 EnvironmentPreference::OnlyVirtual,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })?;
         assert!(
@@ -1570,7 +1567,7 @@ mod tests {
                     EnvironmentPreference::OnlySystem,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             },
         )?;
@@ -1592,7 +1589,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -1607,7 +1604,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })?;
         assert!(
@@ -1621,7 +1618,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })?;
         assert!(
@@ -1650,7 +1647,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -1666,7 +1663,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -1696,7 +1693,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -1712,7 +1709,7 @@ mod tests {
                 EnvironmentPreference::ExplicitSystem,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -1728,7 +1725,7 @@ mod tests {
                 EnvironmentPreference::OnlyVirtual,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -1744,7 +1741,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -1768,7 +1765,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -1783,7 +1780,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -1807,7 +1804,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -1827,7 +1824,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             },
         )??;
@@ -1856,7 +1853,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -1878,7 +1875,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })?;
         assert!(
@@ -1908,7 +1905,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -1924,7 +1921,7 @@ mod tests {
                 EnvironmentPreference::ExplicitSystem,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })?;
         assert!(
@@ -1951,7 +1948,7 @@ mod tests {
                     EnvironmentPreference::ExplicitSystem,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             })
             .unwrap()
@@ -1976,7 +1973,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })?;
         assert!(
@@ -1993,7 +1990,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -2008,7 +2005,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -2034,7 +2031,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -2049,7 +2046,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -2075,7 +2072,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -2102,7 +2099,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -2129,7 +2126,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -2156,7 +2153,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -2183,7 +2180,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -2211,7 +2208,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })?;
         assert!(
@@ -2233,7 +2230,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -2248,7 +2245,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -2274,7 +2271,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -2289,7 +2286,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
         assert_eq!(
@@ -2327,7 +2324,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             })
             .unwrap()
@@ -2345,7 +2342,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             })
             .unwrap()
@@ -2387,7 +2384,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             })
             .unwrap()
@@ -2405,7 +2402,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             })
             .unwrap()
@@ -2442,7 +2439,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             })
             .unwrap()
@@ -2465,7 +2462,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             })
             .unwrap()
@@ -2488,7 +2485,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
-                    PreviewMode::Disabled,
+                    Preview::default(),
                 )
             })
             .unwrap()
@@ -2527,7 +2524,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
 
@@ -2580,7 +2577,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
-                PreviewMode::Disabled,
+                Preview::default(),
             )
         })??;
 
