@@ -1,5 +1,5 @@
-use std::num::NonZeroUsize;
 use std::path::PathBuf;
+use std::{collections::BTreeMap, num::NonZeroUsize};
 
 use url::Url;
 
@@ -12,8 +12,12 @@ use uv_install_wheel::LinkMode;
 use uv_pypi_types::{SchemaConflicts, SupportedEnvironments};
 use uv_python::{PythonDownloads, PythonPreference, PythonVersion};
 use uv_redacted::DisplaySafeUrl;
-use uv_resolver::{AnnotationStyle, ExcludeNewer, ForkStrategy, PrereleaseMode, ResolutionMode};
+use uv_resolver::{
+    AnnotationStyle, ExcludeNewer, ExcludeNewerPackage, ExcludeNewerTimestamp, ForkStrategy,
+    PrereleaseMode, ResolutionMode,
+};
 use uv_torch::TorchMode;
+use uv_workspace::pyproject::ExtraBuildDependencies;
 use uv_workspace::pyproject_mut::AddBoundsKind;
 
 use crate::{FilesystemOptions, Options, PipOptions};
@@ -78,6 +82,7 @@ macro_rules! impl_combine_or {
 impl_combine_or!(AddBoundsKind);
 impl_combine_or!(AnnotationStyle);
 impl_combine_or!(ExcludeNewer);
+impl_combine_or!(ExcludeNewerTimestamp);
 impl_combine_or!(ExportFormat);
 impl_combine_or!(ForkStrategy);
 impl_combine_or!(Index);
@@ -120,6 +125,37 @@ impl<T> Combine for Option<Vec<T>> {
     }
 }
 
+impl<K: Ord, T> Combine for Option<BTreeMap<K, Vec<T>>> {
+    /// Combine two maps of vecs by combining their vecs
+    fn combine(self, other: Option<BTreeMap<K, Vec<T>>>) -> Option<BTreeMap<K, Vec<T>>> {
+        match (self, other) {
+            (Some(mut a), Some(b)) => {
+                for (key, value) in b {
+                    a.entry(key).or_default().extend(value);
+                }
+                Some(a)
+            }
+            (a, b) => a.or(b),
+        }
+    }
+}
+
+impl Combine for Option<ExcludeNewerPackage> {
+    /// Combine two [`ExcludeNewerPackage`] instances by merging them, with the values in `self` taking precedence.
+    fn combine(self, other: Option<ExcludeNewerPackage>) -> Option<ExcludeNewerPackage> {
+        match (self, other) {
+            (Some(mut a), Some(b)) => {
+                // Extend with values from b, but a takes precedence (we don't overwrite existing keys)
+                for (key, value) in b {
+                    a.entry(key).or_insert(value);
+                }
+                Some(a)
+            }
+            (a, b) => a.or(b),
+        }
+    }
+}
+
 impl Combine for Option<ConfigSettings> {
     /// Combine two maps by merging the map in `self` with the map in `other`, if they're both
     /// `Some`.
@@ -153,3 +189,49 @@ impl Combine for Option<serde::de::IgnoredAny> {
         self
     }
 }
+
+impl Combine for ExcludeNewer {
+    fn combine(mut self, other: Self) -> Self {
+        self.global = self.global.combine(other.global);
+
+        if !other.package.is_empty() {
+            if self.package.is_empty() {
+                self.package = other.package;
+            } else {
+                // Merge package-specific timestamps, with self taking precedence
+                for (pkg, timestamp) in &other.package {
+                    self.package.entry(pkg.clone()).or_insert(*timestamp);
+                }
+            }
+        }
+
+        self
+    }
+}
+
+impl Combine for ExtraBuildDependencies {
+    fn combine(mut self, other: Self) -> Self {
+        for (key, value) in other {
+            match self.entry(key) {
+                std::collections::btree_map::Entry::Occupied(mut entry) => {
+                    // Combine the vecs, with self taking precedence
+                    let existing = entry.get_mut();
+                    existing.extend(value);
+                }
+                std::collections::btree_map::Entry::Vacant(entry) => {
+                    entry.insert(value);
+                }
+            }
+        }
+        self
+    }
+}
+
+impl Combine for Option<ExtraBuildDependencies> {
+    fn combine(self, other: Option<ExtraBuildDependencies>) -> Option<ExtraBuildDependencies> {
+        match (self, other) {
+            (Some(a), Some(b)) => Some(a.combine(b)),
+            (a, b) => a.or(b),
+        }
+    }
+}
