@@ -1,6 +1,7 @@
 use anyhow::{Context, Result, anyhow};
 use owo_colors::OwoColorize;
 use std::fmt::Write;
+use std::iter;
 use std::path::{Path, PathBuf};
 use std::process::{Command, Stdio};
 use std::str::FromStr;
@@ -11,7 +12,7 @@ use uv_cache::Cache;
 use uv_cli::AuthorFrom;
 use uv_client::BaseClientBuilder;
 use uv_configuration::{
-    DependencyGroupsWithDefaults, PreviewMode, ProjectBuildBackend, VersionControlError,
+    DependencyGroupsWithDefaults, Preview, ProjectBuildBackend, VersionControlError,
     VersionControlSystem,
 };
 use uv_fs::{CWD, Simplified};
@@ -61,11 +62,8 @@ pub(crate) async fn init(
     no_config: bool,
     cache: &Cache,
     printer: Printer,
-    preview: PreviewMode,
+    preview: Preview,
 ) -> Result<ExitStatus> {
-    if build_backend == Some(ProjectBuildBackend::Uv) && preview.is_disabled() {
-        warn_user_once!("The uv build backend is experimental and may change without warning");
-    }
     match init_kind {
         InitKind::Script => {
             let Some(path) = explicit_path.as_deref() else {
@@ -203,7 +201,7 @@ async fn init_script(
     pin_python: bool,
     package: bool,
     no_config: bool,
-    preview: PreviewMode,
+    preview: Preview,
 ) -> Result<()> {
     if no_workspace {
         warn_user_once!("`--no-workspace` is a no-op for Python scripts, which are standalone");
@@ -298,7 +296,7 @@ async fn init_project(
     no_config: bool,
     cache: &Cache,
     printer: Printer,
-    preview: PreviewMode,
+    preview: Preview,
 ) -> Result<()> {
     // Discover the current workspace, if it exists.
     let workspace_cache = WorkspaceCache::default();
@@ -596,7 +594,6 @@ async fn init_project(
         author_from,
         no_readme,
         package,
-        preview,
     )?;
 
     if let Some(workspace) = workspace {
@@ -724,7 +721,6 @@ impl InitProjectKind {
         author_from: Option<AuthorFrom>,
         no_readme: bool,
         package: bool,
-        preview: PreviewMode,
     ) -> Result<()> {
         match self {
             InitProjectKind::Application => InitProjectKind::init_application(
@@ -739,7 +735,6 @@ impl InitProjectKind {
                 author_from,
                 no_readme,
                 package,
-                preview,
             ),
             InitProjectKind::Library => InitProjectKind::init_library(
                 name,
@@ -753,7 +748,6 @@ impl InitProjectKind {
                 author_from,
                 no_readme,
                 package,
-                preview,
             ),
         }
     }
@@ -772,7 +766,6 @@ impl InitProjectKind {
         author_from: Option<AuthorFrom>,
         no_readme: bool,
         package: bool,
-        preview: PreviewMode,
     ) -> Result<()> {
         fs_err::create_dir_all(path)?;
 
@@ -805,11 +798,7 @@ impl InitProjectKind {
             }
 
             // Add a build system
-            let build_backend = match build_backend {
-                Some(build_backend) => build_backend,
-                None if preview.is_enabled() => ProjectBuildBackend::Uv,
-                None => ProjectBuildBackend::Hatch,
-            };
+            let build_backend = build_backend.unwrap_or(ProjectBuildBackend::Uv);
             pyproject.push('\n');
             pyproject.push_str(&pyproject_build_system(name, build_backend));
             pyproject_build_backend_prerequisites(name, path, build_backend)?;
@@ -859,7 +848,6 @@ impl InitProjectKind {
         author_from: Option<AuthorFrom>,
         no_readme: bool,
         package: bool,
-        preview: PreviewMode,
     ) -> Result<()> {
         if !package {
             return Err(anyhow!("Library projects must be packaged"));
@@ -880,11 +868,7 @@ impl InitProjectKind {
         );
 
         // Always include a build system if the project is packaged.
-        let build_backend = match build_backend {
-            Some(build_backend) => build_backend,
-            None if preview.is_enabled() => ProjectBuildBackend::Uv,
-            None => ProjectBuildBackend::Hatch,
-        };
+        let build_backend = build_backend.unwrap_or(ProjectBuildBackend::Uv);
         pyproject.push('\n');
         pyproject.push_str(&pyproject_build_system(name, build_backend));
         pyproject_build_backend_prerequisites(name, path, build_backend)?;
@@ -961,7 +945,13 @@ fn pyproject_build_system(package: &PackageName, build_backend: ProjectBuildBack
                 min_version.release()[0] == 0,
                 "migrate to major version bumps"
             );
-            let max_version = Version::new([0, min_version.release()[1] + 1]);
+            let max_version = Version::new(
+                [0, min_version.release()[1] + 1]
+                    .into_iter()
+                    // Add trailing zeroes to match the version length, to use the same style
+                    // as `--bounds`.
+                    .chain(iter::repeat_n(0, min_version.release().len() - 2)),
+            );
             indoc::formatdoc! {r#"
                 [build-system]
                 requires = ["uv_build>={min_version},<{max_version}"]
