@@ -1,7 +1,6 @@
 use std::fmt::Write;
 use std::path::{Path, PathBuf};
 use std::str::FromStr;
-use std::sync::Arc;
 use std::vec;
 
 use anyhow::Result;
@@ -12,7 +11,8 @@ use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, FlatIndexClient, RegistryClientBuilder};
 use uv_configuration::{
     BuildOptions, Concurrency, ConfigSettings, Constraints, DependencyGroups, IndexStrategy,
-    KeyringProviderType, NoBinary, NoBuild, PackageConfigSettings, PreviewMode, SourceStrategy,
+    KeyringProviderType, NoBinary, NoBuild, PackageConfigSettings, Preview, PreviewFeatures,
+    SourceStrategy,
 };
 use uv_dispatch::{BuildDispatch, SharedState};
 use uv_distribution_types::Requirement;
@@ -29,6 +29,7 @@ use uv_shell::{Shell, shlex_posix, shlex_windows};
 use uv_types::{AnyErrorBuild, BuildContext, BuildIsolation, BuildStack, HashStrategy};
 use uv_virtualenv::OnExisting;
 use uv_warnings::warn_user;
+use uv_workspace::pyproject::ExtraBuildDependencies;
 use uv_workspace::{DiscoveryOptions, VirtualProject, WorkspaceCache, WorkspaceError};
 
 use crate::commands::ExitStatus;
@@ -75,14 +76,14 @@ pub(crate) async fn venv(
     system_site_packages: bool,
     seed: bool,
     on_existing: OnExisting,
-    exclude_newer: Option<ExcludeNewer>,
+    exclude_newer: ExcludeNewer,
     concurrency: Concurrency,
     no_config: bool,
     no_project: bool,
     cache: &Cache,
     printer: Printer,
     relocatable: bool,
-    preview: PreviewMode,
+    preview: Preview,
 ) -> Result<ExitStatus> {
     let workspace_cache = WorkspaceCache::default();
     let project = if no_project {
@@ -199,7 +200,7 @@ pub(crate) async fn venv(
         path.user_display().cyan()
     )?;
 
-    let upgradeable = preview.is_enabled()
+    let upgradeable = preview.is_enabled(PreviewFeatures::PYTHON_UPGRADE)
         && python_request
             .as_ref()
             .is_none_or(|request| !request.includes_patch());
@@ -224,15 +225,7 @@ pub(crate) async fn venv(
         let interpreter = venv.interpreter();
 
         // Add all authenticated sources to the cache.
-        for index in index_locations.allowed_indexes() {
-            if let Some(credentials) = index.credentials() {
-                let credentials = Arc::new(credentials);
-                uv_auth::store_credentials(index.raw_url(), credentials.clone());
-                if let Some(root_url) = index.root_url() {
-                    uv_auth::store_credentials(&root_url, credentials.clone());
-                }
-            }
-        }
+        index_locations.cache_index_credentials();
 
         // Instantiate a client.
         let client = RegistryClientBuilder::try_from(client_builder)?
@@ -274,7 +267,8 @@ pub(crate) async fn venv(
 
         // Do not allow builds
         let build_options = BuildOptions::new(NoBinary::None, NoBuild::All);
-
+        let extra_build_requires =
+            uv_distribution::ExtraBuildRequires::from_lowered(ExtraBuildDependencies::default());
         // Prep the build context.
         let build_dispatch = BuildDispatch::new(
             &client,
@@ -289,6 +283,7 @@ pub(crate) async fn venv(
             &config_settings,
             &config_settings_package,
             BuildIsolation::Isolated,
+            &extra_build_requires,
             link_mode,
             &build_options,
             &build_hasher,
